unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, UITypes, Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
  ImgList, ieview, iemview, ComCtrls, hyieutils, iexBitmaps, hyiedefs, iesettings, iexRulers,
  System.ImageList, ieopensavedlg, Contnrs, imageenview;

type
  TForm1 = class(TForm)
    ImageEnMView1: TImageEnMView;
    imlDevices: TImageList;
    SaveImageEnDialog1: TSaveImageEnDialog;
    OpenImageEnDialog1: TOpenImageEnDialog;
    ImageEnView1: TImageEnView;
    Panel2: TPanel;
    GroupBox1: TGroupBox;
    DeviceParameters1: TMemo;
    grpFrame: TGroupBox;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    FrameEnabled1: TCheckBox;
    Left1: TEdit;
    Right1: TEdit;
    Top1: TEdit;
    Bottom1: TEdit;
    grpGeneral: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    ResY1: TEdit;
    ResX1: TEdit;
    Colors1: TComboBox;
    Orientation1: TComboBox;
    Rotation1: TComboBox;
    FeederEnabled1: TCheckBox;
    AutoFeed1: TCheckBox;
    DuplexEnabled1: TCheckBox;
    lblSaveToReg: TLabel;
    lblLoadFromReg: TLabel;
    CheckParameters1: TButton;
    StatusBar1: TStatusBar;
    PageControl1: TPageControl;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet1: TTabSheet;
    SelectSource1: TButton;
    Label8: TLabel;
    Twain1: TCheckBox;
    WIA1: TCheckBox;
    DCIM1: TCheckBox;
    Acquire1: TButton;
    Abort1: TButton;
    Sources1: TComboBox;
    VisibleDialog1: TCheckBox;
    EnableLogging1: TCheckBox;
    Open1: TButton;
    Clear1: TButton;
    Rotate1: TButton;
    Sharpen1: TButton;
    SaveAs1: TButton;
    Panel3: TPanel;
    ProgressBar1: TProgressBar;
    Label11: TLabel;
    Exit1: TButton;
    Undo1: TButton;
    Delete1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Abort1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure Sources1Change(Sender: TObject);
    procedure CheckParameters1Click(Sender: TObject);
    procedure Acquire1Click(Sender: TObject);
    procedure SelectSource1Click(Sender: TObject);
    procedure ImageEnIO1Progress(Sender: TObject; per: Integer);
    procedure Sources1DrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure lblLoadFromRegClick(Sender: TObject);
    procedure lblSaveToRegClick(Sender: TObject);
    procedure Rotate1Click(Sender: TObject);
    procedure Sharpen1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure ImageEnMView1FinishWork(Sender: TObject);
    procedure ImageEnMView1ImageSelect(Sender: TObject; idx: Integer);
    procedure Exit1Click(Sender: TObject);
    procedure Undo1Click(Sender: TObject);
    procedure Delete1Click(Sender: TObject);
  private
    { Private declarations }
    AInternalChange: Boolean;
    AAborted: Boolean;
    AFileName: string;
    procedure SaveProperties;
    procedure UpdateControlStatus(bSelectDevice: Boolean = False);
    procedure UpdateStatusbar;
    procedure UpdateGUI;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  ShlObj, iexAcquire, iexRegistryFunctions, iexWindowsFunctions;

const
  Acquire_Orientation: array [ieaoPortrait .. ieaoRotate180] of string = ('Portrait', 'Landscape', 'Rotate 90',
    'Rotate 180');
  Acquire_Rotation: array [iearNone .. iear270CW] of string = ('None', '90 deg. clockwise', '180 deg. clockwise',
    '270 deg. clockwise');
  Acquire_PixelType: array [ieapMonochrome .. ieapOther] of string = ('Monochrome', '8 bit GrayScale', '16 bit GrayScale',
    'Full Color', '16 bit Full Color', 'Other');

{ Global }
function DocumentsFolder: string;
var
  iResult: bool;
  iPath: array [0 .. MAX_PATH] of Char;
begin
  iResult := ShlObj.ShGetSpecialFolderPath(0, iPath, CSIDL_MYDOCUMENTS, False);
  if not iResult then
    raise Exception.Create('Could not find Documents folder location.');
  Result := IncludeTrailingPathDelimiter(iPath);
end;

function AddThousandSeparator(const AString: string; const AChar: char): string;
{ Add a specified thousand separator in the correct location in a string. }
var
  i: Integer; { Loop through separator position }
begin
  Result := AString;
  i := Length(AString) - 2;
  while i > 1 do
  begin
    Insert(AChar, Result, i);
    i := i - 3;
  end;
end;

function AddDefThousandSeparator(const AString: string): string;
{ Add the local default thousand separator in the correct location in a string. }
begin
  Result := AddThousandSeparator(AString, SysUtils.FormatSettings.ThousandSeparator);
end;

function IntegerToString(const i: Int64): string;
{ Add the local default thousand separator in the correct location in a number and return a string. }
begin
  Result := AddThousandSeparator(IntToStr(i), SysUtils.FormatSettings.ThousandSeparator);
end;

// Initialize
procedure TForm1.FormCreate(Sender: TObject);
begin
  AFileName := DocumentsFolder + 'Untitled1.tif';
  ImageEnMView1.SetModernStyling(True, 120, 100);
  ImageEnMView1.Proc.AutoUndo := False;
  ImageEnMView1.Proc.UndoLimit := 25;
end;

// Abort acquisition
procedure TForm1.Abort1Click(Sender: TObject);
begin
  AAborted := True;
  Abort1.Enabled := False;
end;

// Initialize controls
procedure TForm1.FormActivate(Sender: TObject);
var
  i: Integer;
begin
  Screen.Cursor := crHourGlass;
  try
    Colors1.Clear;
    // In newer Delphi versions For.. In.. would be much simpler
    for i := ord(Low(Acquire_PixelType)) to ord(High(Acquire_PixelType)) do
      Colors1.Items.Add(Acquire_PixelType[TIEAcquirePixelType(i)]);
    Orientation1.Clear;
    for i := ord(Low(Acquire_Orientation)) to ord(High(Acquire_Orientation)) do
      Orientation1.Items.Add(Acquire_Orientation[TIEAcquireOrientation(i)]);
    Rotation1.Clear;
    for i := ord(Low(Acquire_Rotation)) to ord(High(Acquire_Rotation)) do
      Rotation1.Items.Add(Acquire_Rotation[TIEAcquireRotation(i)]);
    // Fill sources
    ImageEnMView1.MIO.AcquireParams.FillListWithSources(Sources1.Items);
    UpdateControlStatus(True);
    UpdateGUI;
    UpdateStatusbar;
  finally
    Screen.Cursor := crDefault;
  end;
end;

// User selects scanner
procedure TForm1.Sources1Change(Sender: TObject);
begin
  if AInternalChange then
    exit;
  if Sources1.ItemIndex > -1 then
    ImageEnMView1.MIO.AcquireParams.SetSourceByStr(Sources1.Items[Sources1.ItemIndex]);
  UpdateControlStatus;
end;

// Read parameters again
procedure TForm1.CheckParameters1Click(Sender: TObject);
begin
  SaveProperties;
  // Verify Twain properties...
  if ImageEnMView1.MIO.SelectedAcquireSource.API = ieaTwain then
    ImageEnMView1.MIO.TwainParams.Update;
  // Read back: Some properties may have altered due to TwainParams.Update
  UpdateControlStatus;
end;

// Undo
procedure TForm1.Undo1Click(Sender: TObject);
begin
  ImageEnMView1.Proc.Undo(True);
  ImageEnMView1.Update;
  ImageEnMView1.CopyToIEBitmap(ImageEnMView1.SelectedImage, ImageEnView1.IEBitmap);
  ImageEnView1.Update;
  ImageEnView1.IEBitmap.Modified := ImageEnView1.Proc.CanUndo;
  ImageEnMView1.IEMBitmap.Modified := ImageEnView1.Proc.CanUndo;
  UpdateGUI;
  UpdateStatusbar;
end;

// Set values to controls
procedure TForm1.UpdateControlStatus(bSelectDevice: Boolean = False);
const
  Acquire_Apis: array [ieaTwain .. ieaNone] of string = ('Twain', 'WIA', 'DCIM', 'None');
  Device_Types: array [ieadUnknown .. ieadDrive] of string = ('Unknown', 'Scanner', 'Camera', 'Drive');
  Bool_Types: array [False .. True] of string = ('False', 'True');

  procedure _Add(const sName, sValue: string);
  begin
    DeviceParameters1.Lines.Add(sName + ': ' + sValue);
  end;

  procedure _EnableControls(AControl: TWinControl; bEnable: Boolean);
  var
    i: Integer;
  begin
    AControl.Enabled := bEnable;
    for i := 0 to AControl.ControlCount - 1 do
      AControl.Controls[i].Enabled := bEnable;
  end;

var
  bHaveTrueApiSource: Boolean;
begin
  if bSelectDevice then
  begin
    AInternalChange := True;
    Sources1.ItemIndex := Sources1.Items.IndexOf(AcquireSourceToStr(ImageEnMView1.MIO.AcquireParams.SelectedSource));
    AInternalChange := False;
  end;

  DeviceParameters1.Clear;
  bHaveTrueApiSource := ImageEnMView1.MIO.AcquireParams.SelectedSource.API in [ieaTwain, ieaWIA];
  _EnableControls(grpFrame, bHaveTrueApiSource);
  _EnableControls(grpGeneral, bHaveTrueApiSource);
  CheckParameters1.Enabled := bHaveTrueApiSource;

  if ImageEnMView1.MIO.AcquireParams.SelectedSource.API <> ieaNone then
    with ImageEnMView1.MIO do
    begin
      _Add('Device Name', String(AcquireParams.SelectedSource.Name));
      _Add('API', Acquire_Apis[AcquireParams.SelectedSource.API]);
      _Add('Device Type', Device_Types[AcquireParams.SelectedSource.DeviceType]);

      _Add('Bit Depth', IntToStr(AcquireParams.BitDepth));

      _Add('Physical Size', format('%.2f" x %.2f" (%d x %d px)', [AcquireParams.PhysicalWidth, AcquireParams.PhysicalHeight,
        AcquireParams.PhysicalWidthPixels, AcquireParams.PhysicalHeightPixels]));

      _Add('Feeder Loaded', Bool_Types[AcquireParams.FeederLoaded]);
      _Add('Paper Detectable', Bool_Types[AcquireParams.PaperDetectable]);
      _Add('Duplex Supported', Bool_Types[AcquireParams.DuplexSupported]);

      _Add('Contrast', FloatToStr(AcquireParams.Contrast));
      _Add('Brightness', FloatToStr(AcquireParams.Brightness));
      _Add('Threshold', FloatToStr(AcquireParams.Threshold));

      DeviceParameters1.SelStart := 0;
      DeviceParameters1.SelLength := 0;

      ResX1.Text := FloatToStr(AcquireParams.XResolution);
      ResY1.Text := FloatToStr(AcquireParams.YResolution);

      Colors1.ItemIndex := ord(AcquireParams.PixelType);
      Orientation1.ItemIndex := ord(AcquireParams.Orientation);
      Rotation1.ItemIndex := ord(AcquireParams.Rotation);

      FeederEnabled1.Checked := AcquireParams.FeederEnabled;
      AutoFeed1.Checked := AcquireParams.AutoFeed;
      DuplexEnabled1.Checked := AcquireParams.DuplexEnabled;

      // frame
      Left1.Text := FloatToStr(AcquireParams.AcquireFrameLeft);
      Top1.Text := FloatToStr(AcquireParams.AcquireFrameTop);
      Right1.Text := FloatToStr(AcquireParams.AcquireFrameRight);
      Bottom1.Text := FloatToStr(AcquireParams.AcquireFrameBottom);
      FrameEnabled1.Checked := AcquireParams.AcquireFrameEnabled;

      VisibleDialog1.Checked := AcquireParams.VisibleDialog;

      DuplexEnabled1.Enabled := AcquireParams.DuplexSupported;
      VisibleDialog1.Enabled := AcquireParams.SelectedSource.API <> ieaDCIM;
      if AcquireParams.SelectedSource.API = ieaWIA then
        FrameEnabled1.Enabled := False;
    end;
end;

// Write control values to AcquireParams
procedure TForm1.SaveAs1Click(Sender: TObject);
var
  iExtension: string;
begin
  if SaveImageEnDialog1.Execute then
  begin
    Screen.Cursor := crHourGlass;
    try
      AFileName := SaveImageEnDialog1.FileName;
      iExtension := ExtractFileExt(AFileName);
      if IEExtToFileFormat(iExtension) = ioTIFF then
      begin
        ImageEnMView1.MIO.SaveToFileTIFF(AFileName);
        if ImageEnMView1.MIO.Aborting then
        begin
          MessageDlg('There was an error saving the image.', mtError, [mbOK], 0);
          exit;
        end;
        UpdateGUI;
        UpdateStatusbar;
      end;
    finally
      Screen.Cursor := crDefault;
    end;
  end;
end;

// Save properties
procedure TForm1.SaveProperties;
begin
  with ImageEnMView1.MIO do
  begin
    AcquireParams.XResolution := StrToFloat(ResX1.Text);
    AcquireParams.YResolution := StrToFloat(ResY1.Text);
    AcquireParams.PixelType := TIEAcquirePixelType(Colors1.ItemIndex);
    AcquireParams.Orientation := TIEAcquireOrientation(Orientation1.ItemIndex);
    AcquireParams.Rotation := TIEAcquireRotation(Rotation1.ItemIndex);
    AcquireParams.FeederEnabled := FeederEnabled1.Checked;
    AcquireParams.AutoFeed := AutoFeed1.Checked;
    AcquireParams.DuplexEnabled := DuplexEnabled1.Checked;
    AcquireParams.AcquireFrameLeft := StrToFloat(Left1.Text);
    AcquireParams.AcquireFrameTop := StrToFloat(Top1.Text);
    AcquireParams.AcquireFrameRight := StrToFloat(Right1.Text);
    AcquireParams.AcquireFrameBottom := StrToFloat(Bottom1.Text);
    AcquireParams.AcquireFrameEnabled := FrameEnabled1.Checked;
    AcquireParams.VisibleDialog := VisibleDialog1.Checked;
  end;
end;

// Sharpen
procedure TForm1.Sharpen1Click(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  try
    ImageEnMView1.Proc.SaveUndo('Sharpen ' + IntToStr(ImageEnMView1.Proc.UndoCount));
    ImageEnMView1.Proc.Sharpen;
    ImageEnMView1.Update;
    ImageEnMView1.CopyToIEBitmap(ImageEnMView1.SelectedImage, ImageEnView1.IEBitmap);
    ImageEnView1.Update;
    UpdateGUI;
    UpdateStatusbar;
  finally
    Screen.Cursor := crDefault;
  end;
end;

// Acquire from selected device
procedure TForm1.Acquire1Click(Sender: TObject);
begin
  if ImageEnMView1.MIO.AcquireParams.SelectedSource.API = ieaNone then
  begin
    MessageDlg('You have not specified an acquisition source', mtInformation, [mbOK], 0);
    exit;
  end;
  if (Colors1.Enabled) and (not ImageEnMView1.MIO.AcquireParams.IsPixelTypeSupported(TIEAcquirePixelType(Colors1.ItemIndex)))
  then
  begin
    MessageDlg('The selected color is not supported by your device.', mtError, [mbOK], 0);
    exit;
  end;
  AAborted := False;
  Abort1.Enabled := True;
  Abort1.Visible := True;
  Acquire1.Enabled := False;
  try
    Screen.Cursor := crHourGlass;
    SaveProperties;
    if EnableLogging1.Checked then
      ImageEnMView1.MIO.TwainParams.LogFile := WindowsTempFolder() + 'ietwainlog.txt'
    else
      ImageEnMView1.MIO.TwainParams.LogFile := '';
    ImageEnMView1.MIO.Acquire;
  finally
    Abort1.Visible := False;
    Acquire1.Enabled := True;
    if ImageEnView1.IEBitmap.Width > ImageEnView1.IEBitmap.Height then
      ImageEnView1.FitToWidth
    else
      ImageEnView1.FitToHeight;
    ImageEnMView1.SelectedImage := 0;
    ImageEnMView1.CopyToIEBitmap(ImageEnMView1.SelectedImage, ImageEnView1.IEBitmap);
    ImageEnView1.Update;
    Screen.Cursor := crDefault;
  end;
  UpdateGUI;
  UpdateStatusbar;
  ProgressBar1.Position := 0;
end;

// Prompt the user for an acquire source
procedure TForm1.SelectSource1Click(Sender: TObject);
var
  AcquireApis: TIEAcquireApis;
begin
  AcquireApis := [];
  if Twain1.Checked then
    Include(AcquireApis, ieaTwain);
  if WIA1.Checked then
    Include(AcquireApis, ieaWIA);
  if DCIM1.Checked then
    Include(AcquireApis, ieaDCIM);
  if AcquireApis = [] then
    MessageDlg('You have not specified any acquisition Apis', mtInformation, [mbOK], 0)
  else
  begin
    ImageEnMView1.MIO.SelectAcquireSource(AcquireApis);
    UpdateControlStatus(True);
  end;
end;

// Clear
procedure TForm1.Clear1Click(Sender: TObject);
begin
  AFileName := DocumentsFolder + 'Untitled1.tif';
  ImageEnMView1.Clear;
  ImageEnView1.Clear;
  UpdateGUI;
  UpdateStatusbar;
end;

// Delete
procedure TForm1.Delete1Click(Sender: TObject);
begin
  ImageEnMView1.DeleteImage(ImageEnMView1.SelectedImage);
  UpdateGUI;
  UpdateStatusbar;
end;

// Exit
procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

// Display progress during acquisition
procedure TForm1.ImageEnIO1Progress(Sender: TObject; per: Integer);
const
  Show_Images_As_Acquired = True;
begin
  ProgressBar1.Position := per;
  if Show_Images_As_Acquired then
  begin
    ImageEnMView1.Update;
    Application.ProcessMessages;
    // User has clicked abort?
    if AAborted then
      ImageEnMView1.MIO.Aborting := True;
  end;
end;

// Reset progressbar
procedure TForm1.ImageEnMView1FinishWork(Sender: TObject);
begin
  ProgressBar1.Position := 0;
end;

// Select image
procedure TForm1.ImageEnMView1ImageSelect(Sender: TObject; idx: Integer);
begin
  ImageEnMView1.CopyToIEBitmap(idx, ImageEnView1.IEBitmap);
  if ImageEnView1.IEBitmap.Width > ImageEnView1.IEBitmap.Height then
    ImageEnView1.FitToWidth
  else
    ImageEnView1.FitToHeight;
  ImageEnView1.Update;
  UpdateGUI;
  UpdateStatusbar;
end;

// Handle custom drawing of combobox
procedure TForm1.Sources1DrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
const
  Scanner_Glyph_Index = 0;
  Camera_Glyph_Index = 1;
  Drive_Glyph_Index = 2;
  Device_Glyph_Index = 3;
  Unknown_Glyph_Index = Scanner_Glyph_Index; // Just assume it is a scanner
begin
  // Convert values in cmbSources (which are a coded string) to something human-readable
  DrawAcquireComboListBoxItem(Control, Rect, Sources1.Items[Index], imlDevices, Scanner_Glyph_Index, Camera_Glyph_Index,
    Drive_Glyph_Index, Device_Glyph_Index, Unknown_Glyph_Index);
end;

// Read settings from registry
procedure TForm1.lblLoadFromRegClick(Sender: TObject);
begin
  ImageEnMView1.MIO.AcquireParams.LoadFromRegistry('Software\Xequte\MultiFrame TIFF Scanning\');
  UpdateControlStatus(True);
end;

// Save current settings to registry
procedure TForm1.lblSaveToRegClick(Sender: TObject);
begin
  // write changes to our AcquireParams object
  CheckParameters1Click(nil);
  ImageEnMView1.MIO.AcquireParams.SaveToRegistry('Software\Xequte\MultiFrame TIFF Scanning\');
end;

// UpdateGUI
procedure TForm1.UpdateGUI;
begin
  Clear1.Enabled := ImageEnMView1.ImageCount > 0;
  Rotate1.Enabled := ImageEnMView1.ImageCount > 0;
  Sharpen1.Enabled := ImageEnMView1.ImageCount > 0;
  Undo1.Enabled := ImageEnMView1.Proc.CanUndo;
  Delete1.Enabled := (ImageEnMView1.ImageCount > 0) and (ImageEnMView1.SelectedImage <> -1);
end;

// Update statusbar
procedure TForm1.UpdateStatusbar;
begin
  Caption := 'MultiFrame TIFF Scanning- ' + AFileName;
  StatusBar1.Panels[0].Text := ExtractFileDir(AFileName);
  StatusBar1.Panels[1].Text := ExtractFileName(AFileName);
  StatusBar1.Panels[2].Text := 'Frames: ' + IntToStr(ImageEnMView1.ImageCount);
  StatusBar1.Panels[3].Text := 'Frame: ' + IntToStr(ImageEnMView1.SelectedImage + 1);
  if Assigned(ImageEnMView1.IEBitmap) then
  begin
    StatusBar1.Panels[4].Text := 'Width: ' + IntegerToString(ImageEnMView1.IEBitmap.Width);
    StatusBar1.Panels[5].Text := 'Height: ' + IntegerToString(ImageEnMView1.IEBitmap.Height);
  end;
end;

// Open image
procedure TForm1.Open1Click(Sender: TObject);
var
  iExtension: string;
begin
  if OpenImageEnDialog1.Execute then
  begin
    try
      Screen.Cursor := crHourGlass;
      ImageEnMView1.Clear;
      AFileName := OpenImageEnDialog1.FileName;
      iExtension := ExtractFileExt(AFileName);
      if IEExtToFileFormat(iExtension) = ioTIFF then
      begin
        ImageEnMView1.MIO.LoadFromFile(AFileName);
        if ImageEnMView1.MIO.Aborting then
        begin
          MessageDlg('There was an error loading the image.', mtError, [mbOK], 0);
          exit;
        end;
        ImageEnMView1.SelectedImage := 0;
        ImageEnMView1.CopyToIEBitmap(ImageEnMView1.SelectedImage, ImageEnView1.IEBitmap);
        ImageEnView1.Update;
        if ImageEnView1.IEBitmap.Width > ImageEnView1.IEBitmap.Height then
          ImageEnView1.FitToWidth
        else
          ImageEnView1.FitToHeight;
        UpdateGUI;
        UpdateStatusbar;
      end;
    finally
      Screen.Cursor := crDefault;
    end;
  end;
end;

// Rotate image
procedure TForm1.Rotate1Click(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  try
    ImageEnMView1.Proc.SaveUndo('Rotate ' + IntToStr(ImageEnMView1.Proc.UndoCount));
    ImageEnMView1.Proc.Rotate(90);
    ImageEnMView1.Update;
    ImageEnMView1.CopyToIEBitmap(ImageEnMView1.SelectedImage, ImageEnView1.IEBitmap);
    ImageEnView1.Update;
    UpdateGUI;
    UpdateStatusbar;
  finally
    Screen.Cursor := crDefault;
  end;
end;

end.
